@@ -40,6 +40,7 @@
 
 #include "logging.h"
 #include "glib-helper.h"
+#include "dbus-common.h"
 
 #include "error.h"
 #include "common.h"
@@ -136,10 +137,27 @@ static gboolean bnep_watchdog_cb(GIOChannel *chan, GIOCondition cond,
 
 	if (connection != NULL) {
 		const char *device = nc->dev;
+		gboolean connected = FALSE;
+		const char *property = "";
 		g_dbus_emit_signal(connection, nc->peer->path,
 				NETWORK_PEER_INTERFACE, "Disconnected",
 				DBUS_TYPE_STRING, &device,
 				DBUS_TYPE_INVALID);
+		dbus_connection_emit_property_changed(connection,
+						nc->peer->path,
+						NETWORK_PEER_INTERFACE,
+						"Connected",
+						DBUS_TYPE_BOOLEAN, &connected);
+		dbus_connection_emit_property_changed(connection,
+						nc->peer->path,
+						NETWORK_PEER_INTERFACE,
+						"Device",
+						DBUS_TYPE_STRING, &property);
+		dbus_connection_emit_property_changed(connection,
+						nc->peer->path,
+						NETWORK_PEER_INTERFACE,
+						"UUID",
+						DBUS_TYPE_STRING, &property);
 	}
 
 	info("%s disconnected", nc->dev);
@@ -164,6 +182,7 @@ static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
 	int sk;
 	DBusMessage *reply;
 	const char *pdev, *uuid;
+	gboolean connected;
 
 	if (cond & G_IO_NVAL)
 		return FALSE;
@@ -228,11 +247,21 @@ static gboolean bnep_connect_cb(GIOChannel *chan, GIOCondition cond,
 			DBUS_TYPE_STRING, &pdev,
 			DBUS_TYPE_INVALID);
 
+	connected = TRUE;
 	g_dbus_emit_signal(connection, nc->peer->path,
 			NETWORK_PEER_INTERFACE, "Connected",
 			DBUS_TYPE_STRING, &pdev,
 			DBUS_TYPE_STRING, &uuid,
 			DBUS_TYPE_INVALID);
+	dbus_connection_emit_property_changed(connection, nc->peer->path,
+					NETWORK_PEER_INTERFACE, "Connected",
+					DBUS_TYPE_BOOLEAN, &connected);
+	dbus_connection_emit_property_changed(connection, nc->peer->path,
+					NETWORK_PEER_INTERFACE, "Device",
+					DBUS_TYPE_BOOLEAN, &pdev);
+	dbus_connection_emit_property_changed(connection, nc->peer->path,
+					NETWORK_PEER_INTERFACE, "UUID",
+					DBUS_TYPE_BOOLEAN, &uuid);
 
 	nc->state = CONNECTED;
 
@@ -412,6 +441,51 @@ static DBusMessage *is_connected(DBusConnection *conn,
 						DBUS_TYPE_INVALID);
 }
 
+static DBusMessage *connection_get_properties(DBusConnection *conn,
+					DBusMessage *msg, void *data)
+{
+	struct network_peer *peer = data;
+	struct network_conn *nc;
+	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter dict;
+	dbus_bool_t connected;
+	const char *property;
+	GSList *l;
+
+	dbus_message_iter_init_append(reply, &iter);
+
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+
+	/* Connected */
+	for (l = peer->connections; l; l = l->next) {
+		nc = l->data;
+
+		if (nc->state == CONNECTED)
+			break;
+	}
+	connected = (l != NULL);
+	dbus_message_iter_append_dict_entry(&dict, "Connected",
+						DBUS_TYPE_BOOLEAN, &connected);
+
+	/* Device */
+	property = connected ? nc->dev : "";
+	dbus_message_iter_append_dict_entry(&dict, "Device",
+						DBUS_TYPE_STRING, &property);
+
+	/* UUID */
+	property = connected ? bnep_uuid(nc->id) : "";
+	dbus_message_iter_append_dict_entry(&dict, "UUID",
+						DBUS_TYPE_STRING, &property);
+
+	dbus_message_iter_close_container(&iter, &dict);
+
+	return reply;
+}
+
 static void connection_free(struct network_conn *nc)
 {
 	if (nc->state == CONNECTED) {
@@ -447,13 +521,16 @@ static GDBusMethodTable connection_methods[] = {
 	{ "Connect",		"s",	"s",	connection_connect,
 						G_DBUS_METHOD_FLAG_ASYNC },
 	{ "Disconnect",		"",	"",	connection_disconnect	},
-	{ "IsConnected",	"",	"b",	is_connected		},
+	{ "IsConnected",	"",	"b",	is_connected,
+						G_DBUS_METHOD_FLAG_DEPRECATED },
+	{ "GetProperties",	"",	"a{sv}",connection_get_properties },
 	{ }
 };
 
 static GDBusSignalTable connection_signals[] = {
-	{ "Connected",		"ss"	},
-	{ "Disconnected",	"s"	},
+	{ "Connected",		"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "Disconnected",	"",	G_DBUS_SIGNAL_FLAG_DEPRECATED},
+	{ "PropertyChanged",	"sv"	},
 	{ }
 };
 
